# file generated by setuptools_scm
# don't change, don't track in version control
__version__ = version = '0.0.6.dev0+g81f52a4.d20230202'
__version_tuple__ = version_tuple = (0, 0, 6, 'dev0', 'g81f52a4.d20230202')
